{
 "cells": [
  {
   "attachments": {},
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "# Vision Transformer with Deformable Attention Code"
   ]
  },
  {
   "attachments": {},
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "https://github.com/LeapLabTHU/DAT/blob/566a593daf96efc3df58a1542e60b847b8b6f4ff/models/dat.py"
   ]
  },
  {
   "attachments": {},
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## DAT"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "class DAT(nn.Module):\n",
    "\n",
    "    def __init__(self, img_size=224, patch_size=4, num_classes=1000, expansion=4,\n",
    "                 dim_stem=96, dims=[96, 192, 384, 768], depths=[2, 2, 6, 2], \n",
    "                 heads=[3, 6, 12, 24], \n",
    "                 window_sizes=[7, 7, 7, 7],\n",
    "                 drop_rate=0.0, attn_drop_rate=0.0, drop_path_rate=0.0, \n",
    "                 strides=[-1,-1,-1,-1], offset_range_factor=[1, 2, 3, 4], \n",
    "                 stage_spec=[['L', 'D'], ['L', 'D'], ['L', 'D', 'L', 'D', 'L', 'D'], ['L', 'D']], \n",
    "                 groups=[-1, -1, 3, 6],\n",
    "                 use_pes=[False, False, False, False], \n",
    "                 dwc_pes=[False, False, False, False],\n",
    "                 sr_ratios=[8, 4, 2, 1], \n",
    "                 fixed_pes=[False, False, False, False],\n",
    "                 no_offs=[False, False, False, False],\n",
    "                 ns_per_pts=[4, 4, 4, 4],\n",
    "                 use_dwc_mlps=[False, False, False, False],\n",
    "                 use_conv_patches=False,\n",
    "                 **kwargs):\n",
    "        super().__init__()\n",
    "\n",
    "        self.patch_proj = nn.Sequential(\n",
    "            nn.Conv2d(3, dim_stem, 7, patch_size, 3),\n",
    "            LayerNormProxy(dim_stem)\n",
    "        ) if use_conv_patches else nn.Sequential(\n",
    "            nn.Conv2d(3, dim_stem, patch_size, patch_size, 0),\n",
    "            LayerNormProxy(dim_stem)\n",
    "        ) \n",
    "\n",
    "        img_size = img_size // patch_size\n",
    "        dpr = [x.item() for x in torch.linspace(0, drop_path_rate, sum(depths))]\n",
    "        \n",
    "        self.stages = nn.ModuleList()\n",
    "        for i in range(4):\n",
    "            dim1 = dim_stem if i == 0 else dims[i - 1] * 2\n",
    "            dim2 = dims[i]\n",
    "            self.stages.append(\n",
    "                TransformerStage(img_size, window_sizes[i], ns_per_pts[i],\n",
    "                dim1, dim2, depths[i], stage_spec[i], groups[i], use_pes[i], \n",
    "                sr_ratios[i], heads[i], strides[i], \n",
    "                offset_range_factor[i], i,\n",
    "                dwc_pes[i], no_offs[i], fixed_pes[i],\n",
    "                attn_drop_rate, drop_rate, expansion, drop_rate, \n",
    "                dpr[sum(depths[:i]):sum(depths[:i + 1])],\n",
    "                use_dwc_mlps[i])\n",
    "            )\n",
    "            img_size = img_size // 2\n",
    "\n",
    "        self.down_projs = nn.ModuleList()\n",
    "        for i in range(3):\n",
    "            self.down_projs.append(\n",
    "                nn.Sequential(\n",
    "                    nn.Conv2d(dims[i], dims[i + 1], 3, 2, 1, bias=False),\n",
    "                    LayerNormProxy(dims[i + 1])\n",
    "                ) if use_conv_patches else nn.Sequential(\n",
    "                    nn.Conv2d(dims[i], dims[i + 1], 2, 2, 0, bias=False),\n",
    "                    LayerNormProxy(dims[i + 1])\n",
    "                )\n",
    "            )\n",
    "           \n",
    "        self.cls_norm = LayerNormProxy(dims[-1]) \n",
    "        self.cls_head = nn.Linear(dims[-1], num_classes)\n",
    "        \n",
    "        self.reset_parameters()\n",
    "    \n",
    "    def reset_parameters(self):\n",
    "\n",
    "        for m in self.parameters():\n",
    "            if isinstance(m, (nn.Linear, nn.Conv2d)):\n",
    "                nn.init.kaiming_normal_(m.weight)\n",
    "                nn.init.zeros_(m.bias)\n",
    "    \n",
    "    def forward(self, x):\n",
    "        \n",
    "        x = self.patch_proj(x)\n",
    "        positions = []\n",
    "        references = []\n",
    "        for i in range(4):\n",
    "            x, pos, ref = self.stages[i](x)\n",
    "            if i < 3:\n",
    "                x = self.down_projs[i](x)\n",
    "            positions.append(pos)\n",
    "            references.append(ref)\n",
    "        x = self.cls_norm(x)\n",
    "        x = F.adaptive_avg_pool2d(x, 1)\n",
    "        x = torch.flatten(x, 1)\n",
    "        x = self.cls_head(x)\n",
    "        \n",
    "        return x, positions, references"
   ]
  },
  {
   "attachments": {},
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## TransformerStage"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "class TransformerStage(nn.Module):\n",
    "\n",
    "    def __init__(self, fmap_size, window_size, ns_per_pt,\n",
    "                 dim_in, dim_embed, depths, stage_spec, n_groups, \n",
    "                 use_pe, sr_ratio, \n",
    "                 heads, stride, offset_range_factor, stage_idx,\n",
    "                 dwc_pe, no_off, fixed_pe,\n",
    "                 attn_drop, proj_drop, expansion, drop, drop_path_rate, use_dwc_mlp):\n",
    "\n",
    "        super().__init__()\n",
    "        fmap_size = to_2tuple(fmap_size)\n",
    "        self.depths = depths\n",
    "        hc = dim_embed // heads\n",
    "        assert dim_embed == heads * hc\n",
    "        self.proj = nn.Conv2d(dim_in, dim_embed, 1, 1, 0) if dim_in != dim_embed else nn.Identity()\n",
    "\n",
    "        self.layer_norms = nn.ModuleList(\n",
    "            [LayerNormProxy(dim_embed) for _ in range(2 * depths)]\n",
    "        )\n",
    "        self.mlps = nn.ModuleList(\n",
    "            [\n",
    "                TransformerMLPWithConv(dim_embed, expansion, drop) \n",
    "                if use_dwc_mlp else TransformerMLP(dim_embed, expansion, drop)\n",
    "                for _ in range(depths)\n",
    "            ]\n",
    "        )\n",
    "        self.attns = nn.ModuleList()\n",
    "        self.drop_path = nn.ModuleList()\n",
    "        for i in range(depths):\n",
    "            if stage_spec[i] == 'L':\n",
    "                self.attns.append(\n",
    "                    LocalAttention(dim_embed, heads, window_size, attn_drop, proj_drop)\n",
    "                )\n",
    "            elif stage_spec[i] == 'D':\n",
    "                self.attns.append(\n",
    "                    DAttentionBaseline(fmap_size, fmap_size, heads, \n",
    "                    hc, n_groups, attn_drop, proj_drop, \n",
    "                    stride, offset_range_factor, use_pe, dwc_pe, \n",
    "                    no_off, fixed_pe, stage_idx)\n",
    "                )\n",
    "            elif stage_spec[i] == 'S':\n",
    "                shift_size = math.ceil(window_size / 2)\n",
    "                self.attns.append(\n",
    "                    ShiftWindowAttention(dim_embed, heads, window_size, attn_drop, proj_drop, shift_size, fmap_size)\n",
    "                )\n",
    "            else:\n",
    "                raise NotImplementedError(f'Spec: {stage_spec[i]} is not supported.')\n",
    "            \n",
    "            self.drop_path.append(DropPath(drop_path_rate[i]) if drop_path_rate[i] > 0.0 else nn.Identity())\n",
    "        \n",
    "    def forward(self, x):\n",
    "        \n",
    "        x = self.proj(x)\n",
    "        \n",
    "        positions = []\n",
    "        references = []\n",
    "        for d in range(self.depths):\n",
    "\n",
    "            x0 = x\n",
    "            x, pos, ref = self.attns[d](self.layer_norms[2 * d](x))\n",
    "            x = self.drop_path[d](x) + x0\n",
    "            x0 = x\n",
    "            x = self.mlps[d](self.layer_norms[2 * d + 1](x))\n",
    "            x = self.drop_path[d](x) + x0\n",
    "            positions.append(pos)\n",
    "            references.append(ref)\n",
    "\n",
    "        return x, positions, references"
   ]
  },
  {
   "attachments": {},
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## TransformerMLPWithConv"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "class TransformerMLPWithConv(nn.Module):\n",
    "\n",
    "    def __init__(self, channels, expansion, drop):\n",
    "        \n",
    "        super().__init__()\n",
    "        \n",
    "        self.dim1 = channels\n",
    "        self.dim2 = channels * expansion\n",
    "        self.linear1 = nn.Conv2d(self.dim1, self.dim2, 1, 1, 0)\n",
    "        self.drop1 = nn.Dropout(drop, inplace=True)\n",
    "        self.act = nn.GELU()\n",
    "        self.linear2 = nn.Conv2d(self.dim2, self.dim1, 1, 1, 0) \n",
    "        self.drop2 = nn.Dropout(drop, inplace=True)\n",
    "        self.dwc = nn.Conv2d(self.dim2, self.dim2, 3, 1, 1, groups=self.dim2)\n",
    "    \n",
    "    def forward(self, x):\n",
    "        \n",
    "        x = self.drop1(self.act(self.dwc(self.linear1(x))))\n",
    "        x = self.drop2(self.linear2(x))\n",
    "        \n",
    "        return "
   ]
  },
  {
   "attachments": {},
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## TransformerMLP"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "class TransformerMLP(nn.Module):\n",
    "\n",
    "    def __init__(self, channels, expansion, drop):\n",
    "        \n",
    "        super().__init__()\n",
    "        \n",
    "        self.dim1 = channels\n",
    "        self.dim2 = channels * expansion\n",
    "        self.chunk = nn.Sequential()\n",
    "        self.chunk.add_module('linear1', nn.Linear(self.dim1, self.dim2))\n",
    "        self.chunk.add_module('act', nn.GELU())\n",
    "        self.chunk.add_module('drop1', nn.Dropout(drop, inplace=True))\n",
    "        self.chunk.add_module('linear2', nn.Linear(self.dim2, self.dim1))\n",
    "        self.chunk.add_module('drop2', nn.Dropout(drop, inplace=True))\n",
    "    \n",
    "    def forward(self, x):\n",
    "\n",
    "        _, _, H, W = x.size()\n",
    "        x = einops.rearrange(x, 'b c h w -> b (h w) c')\n",
    "        x = self.chunk(x)\n",
    "        x = einops.rearrange(x, 'b (h w) c -> b c h w', h=H, w=W)\n",
    "        return "
   ]
  },
  {
   "attachments": {},
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## LocalAttention"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "class LocalAttention(nn.Module):\n",
    "\n",
    "    def __init__(self, dim, heads, window_size, attn_drop, proj_drop):\n",
    "        \n",
    "        super().__init__()\n",
    "\n",
    "        window_size = to_2tuple(window_size)\n",
    "\n",
    "        self.proj_qkv = nn.Linear(dim, 3 * dim)\n",
    "        self.heads = heads\n",
    "        assert dim % heads == 0\n",
    "        head_dim = dim // heads\n",
    "        self.scale = head_dim ** -0.5\n",
    "        self.proj_out = nn.Linear(dim, dim)\n",
    "        self.window_size = window_size\n",
    "        self.proj_drop = nn.Dropout(proj_drop, inplace=True)\n",
    "        self.attn_drop = nn.Dropout(attn_drop, inplace=True)\n",
    "\n",
    "        Wh, Ww = self.window_size\n",
    "        self.relative_position_bias_table = nn.Parameter(\n",
    "            torch.zeros((2 * Wh - 1) * (2 * Ww - 1), heads)\n",
    "        )\n",
    "        trunc_normal_(self.relative_position_bias_table, std=0.01)\n",
    "\n",
    "        coords_h = torch.arange(self.window_size[0])\n",
    "        coords_w = torch.arange(self.window_size[1])\n",
    "        coords = torch.stack(torch.meshgrid([coords_h, coords_w]))  # 2, Wh, Ww\n",
    "        coords_flatten = torch.flatten(coords, 1)  # 2, Wh*Ww\n",
    "        relative_coords = coords_flatten[:, :, None] - coords_flatten[:, None, :]  # 2, Wh*Ww, Wh*Ww\n",
    "        relative_coords = relative_coords.permute(1, 2, 0).contiguous()  # Wh*Ww, Wh*Ww, 2\n",
    "        relative_coords[:, :, 0] += self.window_size[0] - 1  # shift to start from 0\n",
    "        relative_coords[:, :, 1] += self.window_size[1] - 1\n",
    "        relative_coords[:, :, 0] *= 2 * self.window_size[1] - 1\n",
    "        relative_position_index = relative_coords.sum(-1)  # Wh*Ww, Wh*Ww\n",
    "        self.register_buffer(\"relative_position_index\", relative_position_index)\n",
    "\n",
    "    def forward(self, x, mask=None):\n",
    "\n",
    "        B, C, H, W = x.size()\n",
    "        r1, r2 = H // self.window_size[0], W // self.window_size[1]\n",
    "        \n",
    "        x_total = einops.rearrange(x, 'b c (r1 h1) (r2 w1) -> b (r1 r2) (h1 w1) c', h1=self.window_size[0], w1=self.window_size[1]) # B x Nr x Ws x C\n",
    "        \n",
    "        x_total = einops.rearrange(x_total, 'b m n c -> (b m) n c')\n",
    "\n",
    "        qkv = self.proj_qkv(x_total) # B' x N x 3C\n",
    "        q, k, v = torch.chunk(qkv, 3, dim=2)\n",
    "\n",
    "        q = q * self.scale\n",
    "        q, k, v = [einops.rearrange(t, 'b n (h c1) -> b h n c1', h=self.heads) for t in [q, k, v]]\n",
    "        attn = torch.einsum('b h m c, b h n c -> b h m n', q, k)\n",
    "\n",
    "        relative_position_bias = self.relative_position_bias_table[self.relative_position_index.view(-1)].view(\n",
    "            self.window_size[0] * self.window_size[1], self.window_size[0] * self.window_size[1], -1)  # Wh*Ww,Wh*Ww,nH\n",
    "        relative_position_bias = relative_position_bias.permute(2, 0, 1).contiguous()  # nH, Wh*Ww, Wh*Ww\n",
    "        attn_bias = relative_position_bias\n",
    "        attn = attn + attn_bias.unsqueeze(0)\n",
    "\n",
    "        if mask is not None:\n",
    "            # attn : (b * nW) h w w\n",
    "            # mask : nW ww ww\n",
    "            nW, ww, _ = mask.size()\n",
    "            attn = einops.rearrange(attn, '(b n) h w1 w2 -> b n h w1 w2', n=nW, h=self.heads, w1=ww, w2=ww) + mask.reshape(1, nW, 1, ww, ww)\n",
    "            attn = einops.rearrange(attn, 'b n h w1 w2 -> (b n) h w1 w2')\n",
    "        attn = self.attn_drop(attn.softmax(dim=3))\n",
    "\n",
    "        x = torch.einsum('b h m n, b h n c -> b h m c', attn, v)\n",
    "        x = einops.rearrange(x, 'b h n c1 -> b n (h c1)')\n",
    "        x = self.proj_drop(self.proj_out(x)) # B' x N x C\n",
    "        x = einops.rearrange(x, '(b r1 r2) (h1 w1) c -> b c (r1 h1) (r2 w1)', r1=r1, r2=r2, h1=self.window_size[0], w1=self.window_size[1]) # B x C x H x W\n",
    "        \n",
    "        return x, None, None"
   ]
  },
  {
   "attachments": {},
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## DAttentionBaseline"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "class DAttentionBaseline(nn.Module):\n",
    "\n",
    "    def __init__(\n",
    "        self, q_size, kv_size, n_heads, n_head_channels, n_groups,\n",
    "        attn_drop, proj_drop, stride, \n",
    "        offset_range_factor, use_pe, dwc_pe,\n",
    "        no_off, fixed_pe, stage_idx\n",
    "    ):\n",
    "\n",
    "        super().__init__()\n",
    "        self.dwc_pe = dwc_pe\n",
    "        self.n_head_channels = n_head_channels\n",
    "        self.scale = self.n_head_channels ** -0.5\n",
    "        self.n_heads = n_heads\n",
    "        self.q_h, self.q_w = q_size\n",
    "        self.kv_h, self.kv_w = kv_size\n",
    "        self.nc = n_head_channels * n_heads\n",
    "        self.n_groups = n_groups\n",
    "        self.n_group_channels = self.nc // self.n_groups\n",
    "        self.n_group_heads = self.n_heads // self.n_groups\n",
    "        self.use_pe = use_pe\n",
    "        self.fixed_pe = fixed_pe\n",
    "        self.no_off = no_off\n",
    "        self.offset_range_factor = offset_range_factor\n",
    "        \n",
    "        ksizes = [9, 7, 5, 3]\n",
    "        kk = ksizes[stage_idx]\n",
    "\n",
    "        self.conv_offset = nn.Sequential(\n",
    "            nn.Conv2d(self.n_group_channels, self.n_group_channels, kk, stride, kk//2, groups=self.n_group_channels),\n",
    "            LayerNormProxy(self.n_group_channels),\n",
    "            nn.GELU(),\n",
    "            nn.Conv2d(self.n_group_channels, 2, 1, 1, 0, bias=False)\n",
    "        )\n",
    "\n",
    "        self.proj_q = nn.Conv2d(\n",
    "            self.nc, self.nc,\n",
    "            kernel_size=1, stride=1, padding=0\n",
    "        )\n",
    "\n",
    "        self.proj_k = nn.Conv2d(\n",
    "            self.nc, self.nc,\n",
    "            kernel_size=1, stride=1, padding=0\n",
    "        )\n",
    "\n",
    "        self.proj_v = nn.Conv2d(\n",
    "            self.nc, self.nc,\n",
    "            kernel_size=1, stride=1, padding=0\n",
    "        )\n",
    "\n",
    "        self.proj_out = nn.Conv2d(\n",
    "            self.nc, self.nc,\n",
    "            kernel_size=1, stride=1, padding=0\n",
    "        )\n",
    "\n",
    "        self.proj_drop = nn.Dropout(proj_drop, inplace=True)\n",
    "        self.attn_drop = nn.Dropout(attn_drop, inplace=True)\n",
    "\n",
    "        if self.use_pe:\n",
    "            if self.dwc_pe:\n",
    "                self.rpe_table = nn.Conv2d(self.nc, self.nc, \n",
    "                                           kernel_size=3, stride=1, padding=1, groups=self.nc)\n",
    "            elif self.fixed_pe:\n",
    "                self.rpe_table = nn.Parameter(\n",
    "                    torch.zeros(self.n_heads, self.q_h * self.q_w, self.kv_h * self.kv_w)\n",
    "                )\n",
    "                trunc_normal_(self.rpe_table, std=0.01)\n",
    "            else:\n",
    "                self.rpe_table = nn.Parameter(\n",
    "                    torch.zeros(self.n_heads, self.kv_h * 2 - 1, self.kv_w * 2 - 1)\n",
    "                )\n",
    "                trunc_normal_(self.rpe_table, std=0.01)\n",
    "        else:\n",
    "            self.rpe_table = None\n",
    "    \n",
    "    @torch.no_grad()\n",
    "    def _get_ref_points(self, H_key, W_key, B, dtype, device):\n",
    "        \n",
    "        ref_y, ref_x = torch.meshgrid(\n",
    "            torch.linspace(0.5, H_key - 0.5, H_key, dtype=dtype, device=device), \n",
    "            torch.linspace(0.5, W_key - 0.5, W_key, dtype=dtype, device=device)\n",
    "        )\n",
    "        ref = torch.stack((ref_y, ref_x), -1)\n",
    "        ref[..., 1].div_(W_key).mul_(2).sub_(1)\n",
    "        ref[..., 0].div_(H_key).mul_(2).sub_(1)\n",
    "        ref = ref[None, ...].expand(B * self.n_groups, -1, -1, -1) # B * g H W 2\n",
    "        \n",
    "        return ref\n",
    "\n",
    "    def forward(self, x):\n",
    "\n",
    "        B, C, H, W = x.size()\n",
    "        dtype, device = x.dtype, x.device\n",
    "        \n",
    "        q = self.proj_q(x)\n",
    "        q_off = einops.rearrange(q, 'b (g c) h w -> (b g) c h w', g=self.n_groups, c=self.n_group_channels)\n",
    "        offset = self.conv_offset(q_off) # B * g 2 Hg Wg\n",
    "        Hk, Wk = offset.size(2), offset.size(3)\n",
    "        n_sample = Hk * Wk\n",
    "        \n",
    "        if self.offset_range_factor > 0:\n",
    "            offset_range = torch.tensor([1.0 / Hk, 1.0 / Wk], device=device).reshape(1, 2, 1, 1)\n",
    "            offset = offset.tanh().mul(offset_range).mul(self.offset_range_factor)\n",
    "            \n",
    "        offset = einops.rearrange(offset, 'b p h w -> b h w p')\n",
    "        reference = self._get_ref_points(Hk, Wk, B, dtype, device)\n",
    "            \n",
    "        if self.no_off:\n",
    "            offset = offset.fill(0.0)\n",
    "            \n",
    "        if self.offset_range_factor >= 0:\n",
    "            pos = offset + reference\n",
    "        else:\n",
    "            pos = (offset + reference).tanh()\n",
    "        \n",
    "        x_sampled = F.grid_sample(\n",
    "            input=x.reshape(B * self.n_groups, self.n_group_channels, H, W), \n",
    "            grid=pos[..., (1, 0)], # y, x -> x, y\n",
    "            mode='bilinear', align_corners=True) # B * g, Cg, Hg, Wg\n",
    "            \n",
    "        x_sampled = x_sampled.reshape(B, C, 1, n_sample)\n",
    "\n",
    "        q = q.reshape(B * self.n_heads, self.n_head_channels, H * W)\n",
    "        k = self.proj_k(x_sampled).reshape(B * self.n_heads, self.n_head_channels, n_sample)\n",
    "        v = self.proj_v(x_sampled).reshape(B * self.n_heads, self.n_head_channels, n_sample)\n",
    "        \n",
    "        attn = torch.einsum('b c m, b c n -> b m n', q, k) # B * h, HW, Ns\n",
    "        attn = attn.mul(self.scale)\n",
    "        \n",
    "        if self.use_pe:\n",
    "            \n",
    "            if self.dwc_pe:\n",
    "                residual_lepe = self.rpe_table(q.reshape(B, C, H, W)).reshape(B * self.n_heads, self.n_head_channels, H * W)\n",
    "            elif self.fixed_pe:\n",
    "                rpe_table = self.rpe_table\n",
    "                attn_bias = rpe_table[None, ...].expand(B, -1, -1, -1)\n",
    "                attn = attn + attn_bias.reshape(B * self.n_heads, H * W, self.n_sample)\n",
    "            else:\n",
    "                rpe_table = self.rpe_table\n",
    "                rpe_bias = rpe_table[None, ...].expand(B, -1, -1, -1)\n",
    "                \n",
    "                q_grid = self._get_ref_points(H, W, B, dtype, device)\n",
    "                \n",
    "                displacement = (q_grid.reshape(B * self.n_groups, H * W, 2).unsqueeze(2) - pos.reshape(B * self.n_groups, n_sample, 2).unsqueeze(1)).mul(0.5)\n",
    "                \n",
    "                attn_bias = F.grid_sample(\n",
    "                    input=rpe_bias.reshape(B * self.n_groups, self.n_group_heads, 2 * H - 1, 2 * W - 1),\n",
    "                    grid=displacement[..., (1, 0)],\n",
    "                    mode='bilinear', align_corners=True\n",
    "                ) # B * g, h_g, HW, Ns\n",
    "                \n",
    "                attn_bias = attn_bias.reshape(B * self.n_heads, H * W, n_sample)\n",
    "                \n",
    "                attn = attn + attn_bias\n",
    "\n",
    "        attn = F.softmax(attn, dim=2)\n",
    "        attn = self.attn_drop(attn)\n",
    "        \n",
    "        out = torch.einsum('b m n, b c n -> b c m', attn, v)\n",
    "        \n",
    "        if self.use_pe and self.dwc_pe:\n",
    "            out = out + residual_lepe\n",
    "        out = out.reshape(B, C, H, W)\n",
    "        \n",
    "        y = self.proj_drop(self.proj_out(out))\n",
    "        \n",
    "        return y, pos.reshape(B, self.n_groups, Hk, Wk, 2), reference.reshape(B, self.n_groups, Hk, Wk, 2)"
   ]
  },
  {
   "attachments": {},
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## ShiftWindowAttention"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "class ShiftWindowAttention(LocalAttention):\n",
    "\n",
    "    def __init__(self, dim, heads, window_size, attn_drop, proj_drop, shift_size, fmap_size):\n",
    "        \n",
    "        super().__init__(dim, heads, window_size, attn_drop, proj_drop)\n",
    "\n",
    "        self.fmap_size = to_2tuple(fmap_size)\n",
    "        self.shift_size = shift_size\n",
    "\n",
    "        assert 0 < self.shift_size < min(self.window_size), \"wrong shift size.\"\n",
    "\n",
    "        img_mask = torch.zeros(*self.fmap_size)  # H W\n",
    "        h_slices = (slice(0, -self.window_size[0]),\n",
    "                    slice(-self.window_size[0], -self.shift_size),\n",
    "                    slice(-self.shift_size, None))\n",
    "        w_slices = (slice(0, -self.window_size[1]),\n",
    "                    slice(-self.window_size[1], -self.shift_size),\n",
    "                    slice(-self.shift_size, None))\n",
    "        cnt = 0\n",
    "        for h in h_slices:\n",
    "            for w in w_slices:\n",
    "                img_mask[h, w] = cnt\n",
    "                cnt += 1\n",
    "        mask_windows = einops.rearrange(img_mask, '(r1 h1) (r2 w1) -> (r1 r2) (h1 w1)', h1=self.window_size[0],w1=self.window_size[1])\n",
    "        attn_mask = mask_windows.unsqueeze(1) - mask_windows.unsqueeze(2) # nW ww ww\n",
    "        attn_mask = attn_mask.masked_fill(attn_mask != 0, float(-100.0)).masked_fill(attn_mask == 0, float(0.0))\n",
    "        self.register_buffer(\"attn_mask\", attn_mask)\n",
    "      \n",
    "    def forward(self, x):\n",
    "\n",
    "        shifted_x = torch.roll(x, shifts=(-self.shift_size, -self.shift_size), dims=(2, 3))\n",
    "        sw_x, _, _ = super().forward(shifted_x, self.attn_mask)\n",
    "        x = torch.roll(sw_x, shifts=(self.shift_size, self.shift_size), dims=(2, 3))\n",
    "\n",
    "        return x, None, None"
   ]
  },
  {
   "attachments": {},
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "\n",
    "---\n",
    "\n",
    "Author by `박민식`  \n",
    "Edit by `김주영`"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "name": "python",
   "version": "3.10.8 (main, Oct 13 2022, 09:48:40) [Clang 14.0.0 (clang-1400.0.29.102)]"
  },
  "orig_nbformat": 4,
  "vscode": {
   "interpreter": {
    "hash": "b0fa6594d8f4cbf19f97940f81e996739fb7646882a419484c72d19e05852a7e"
   }
  }
 },
 "nbformat": 4,
 "nbformat_minor": 2
}
